diff --git a/build.xml b/build.xml
index d56d271d8..6ce182a78 100644
--- a/build.xml
+++ b/build.xml
@@ -37,6 +37,19 @@
+
+
+
+
+
+
+
+
+
+
+
@@ -77,6 +90,7 @@
+
diff --git a/lib/Doctrine/ORM/EntityManager.php b/lib/Doctrine/ORM/EntityManager.php
index b5ed0550f..6efae2000 100644
--- a/lib/Doctrine/ORM/EntityManager.php
+++ b/lib/Doctrine/ORM/EntityManager.php
@@ -302,11 +302,11 @@ class EntityManager
/**
* Gets a reference to the entity identified by the given type and identifier
- * without actually loading it. Only the identifier of the returned entity
- * will be populated.
- *
- * NOTE: There is currently no magic proxying in place, that means the full state
- * of the entity will not be loaded upon accessing it.
+ * without actually loading it.
+ *
+ * If partial objects are allowed, this method will return a partial object that only
+ * has its identifier populated. Otherwise a proxy is returned that automatically
+ * loads itself on first access.
*
* @return object The entity reference.
*/
diff --git a/lib/Doctrine/ORM/Internal/CommitOrderCalculator.php b/lib/Doctrine/ORM/Internal/CommitOrderCalculator.php
index 6f12cc871..cc1431949 100644
--- a/lib/Doctrine/ORM/Internal/CommitOrderCalculator.php
+++ b/lib/Doctrine/ORM/Internal/CommitOrderCalculator.php
@@ -56,7 +56,7 @@ class CommitOrderCalculator
* Uses a depth-first search (DFS) to traverse the graph.
* The desired topological sorting is the reverse postorder of these searches.
*
- * @return array The list of ordered items. These are the items wrapped in the nodes.
+ * @return array The list of ordered classes.
*/
public function getCommitOrder()
{
@@ -65,7 +65,7 @@ class CommitOrderCalculator
if ($nodeCount == 0) {
return array();
} else if ($nodeCount == 1) {
- return $this->_classes;
+ return array(0 => array_pop($this->_classes));
}
// Init
@@ -102,7 +102,6 @@ class CommitOrderCalculator
}
$this->_nodeStates[$node->name] = self::VISITED;
-
$this->_sorted[] = $node;
}
diff --git a/lib/Doctrine/ORM/Internal/Hydration/ObjectHydrator.php b/lib/Doctrine/ORM/Internal/Hydration/ObjectHydrator.php
index ace2efde8..4b4801960 100644
--- a/lib/Doctrine/ORM/Internal/Hydration/ObjectHydrator.php
+++ b/lib/Doctrine/ORM/Internal/Hydration/ObjectHydrator.php
@@ -363,9 +363,8 @@ class ObjectHydrator extends AbstractHydrator
if ($this->_rsm->isMixed) {
$element = array($key => $element);
$result[] = $element;
+ $this->_identifierMap[$dqlAlias][$id[$dqlAlias]] = $this->_resultCounter;
++$this->_resultCounter;
- end($result);
- $this->_identifierMap[$dqlAlias][$id[$dqlAlias]] = key($result);
} else {
$result[$key] = $element;
$this->_identifierMap[$dqlAlias][$id[$dqlAlias]] = $key;
@@ -373,11 +372,10 @@ class ObjectHydrator extends AbstractHydrator
} else {
if ($this->_rsm->isMixed) {
$element = array(0 => $element);
- ++$this->_resultCounter;
}
$result[] = $element;
- end($result);
- $this->_identifierMap[$dqlAlias][$id[$dqlAlias]] = key($result);
+ $this->_identifierMap[$dqlAlias][$id[$dqlAlias]] = $this->_resultCounter;
+ ++$this->_resultCounter;
}
// Update result pointer
diff --git a/lib/Doctrine/ORM/Mapping/ClassMetadataFactory.php b/lib/Doctrine/ORM/Mapping/ClassMetadataFactory.php
index 49e8ad307..e620569a7 100644
--- a/lib/Doctrine/ORM/Mapping/ClassMetadataFactory.php
+++ b/lib/Doctrine/ORM/Mapping/ClassMetadataFactory.php
@@ -86,6 +86,11 @@ class ClassMetadataFactory
{
return $this->_cacheDriver;
}
+
+ public function getLoadedMetadata()
+ {
+ return $this->_loadedMetadata;
+ }
/**
* Gets the class metadata descriptor for a class.
diff --git a/lib/Doctrine/ORM/Mapping/Driver/AbstractFileDriver.php b/lib/Doctrine/ORM/Mapping/Driver/AbstractFileDriver.php
index 573c3bdaf..96e2e9f9c 100644
--- a/lib/Doctrine/ORM/Mapping/Driver/AbstractFileDriver.php
+++ b/lib/Doctrine/ORM/Mapping/Driver/AbstractFileDriver.php
@@ -91,9 +91,9 @@ abstract class AbstractFileDriver implements Driver
* documents and operates in the specified operating mode.
*
* @param string|array $paths One or multiple paths where mapping documents can be found.
- * @param integer $mode The operating mode. Either PRELOAD (default) or FILE_PER_CLASS.
+ * @param integer $mode The operating mode. Either PRELOAD or FILE_PER_CLASS (default).
*/
- public function __construct($paths, $mode = self::PRELOAD)
+ public function __construct($paths, $mode = self::FILE_PER_CLASS)
{
$this->_paths = $paths;
$this->_mode = $mode;
diff --git a/lib/Doctrine/ORM/Mapping/OneToOneMapping.php b/lib/Doctrine/ORM/Mapping/OneToOneMapping.php
index 7368a4845..7b72af617 100644
--- a/lib/Doctrine/ORM/Mapping/OneToOneMapping.php
+++ b/lib/Doctrine/ORM/Mapping/OneToOneMapping.php
@@ -120,6 +120,10 @@ class OneToOneMapping extends AssociationMapping
$this->orphanRemoval = isset($mapping['orphanRemoval']) ?
(bool) $mapping['orphanRemoval'] : false;
+ /*if ($this->isOptional) {
+ $this->fetchMode = self::FETCH_EAGER;
+ }*/
+
return $mapping;
}
diff --git a/lib/Doctrine/ORM/Query.php b/lib/Doctrine/ORM/Query.php
index 107cc41c2..d69cda88a 100644
--- a/lib/Doctrine/ORM/Query.php
+++ b/lib/Doctrine/ORM/Query.php
@@ -172,6 +172,7 @@ final class Query extends AbstractQuery
// Calculate hash for dql query.
// TODO: Probably need to include query hints in hash calculation, because query hints
// can have influence on the SQL.
+ // TODO: Include _maxResults and _firstResult in hash calculation
$hash = md5($this->getDql() . 'DOCTRINE_QUERY_CACHE_SALT');
$cached = ($this->_expireQueryCache) ? false : $queryCache->fetch($hash);
diff --git a/lib/Doctrine/ORM/Tools/Cli.php b/lib/Doctrine/ORM/Tools/Cli.php
index 012623567..0ad79cf26 100644
--- a/lib/Doctrine/ORM/Tools/Cli.php
+++ b/lib/Doctrine/ORM/Tools/Cli.php
@@ -22,9 +22,9 @@
namespace Doctrine\ORM\Tools;
use Doctrine\Common\Util\Inflector,
- Doctrine\ORM\Tools\Cli\AbstractPrinter,
- Doctrine\ORM\Tools\Cli\AbstractTask,
- Doctrine\ORM\Tools\Cli\Printer;
+ Doctrine\ORM\Tools\Cli\Printers\AbstractPrinter,
+ Doctrine\ORM\Tools\Cli\Tasks\AbstractTask,
+ Doctrine\ORM\Tools\Cli\Printers\AnsiColorPrinter;
/**
* Generic CLI Runner of Tasks
@@ -83,14 +83,16 @@ class Cli
public function __construct(AbstractPrinter $printer = null)
{
//$this->_printer = new Printer\Normal();
- $this->_printer = $printer ?: new Printer\AnsiColor();
+ $this->_printer = $printer ?: new AnsiColorPrinter;
// Include core tasks
- $ns = 'Doctrine\ORM\Tools\Cli\Task';
+ $ns = 'Doctrine\ORM\Tools\Cli\Tasks';
$this->addTasks(array(
- 'help' => $ns . '\Help',
- 'version' => $ns . '\Version',
+ 'help' => $ns . '\HelpTask',
+ 'version' => $ns . '\VersionTask',
+ 'schema-tool' => $ns . '\SchemaToolTask',
+ 'run-sql' => $ns . '\RunSqlTask'
));
}
@@ -146,12 +148,12 @@ class Cli
// Automatically prepend 'help' task if:
// 1- No arguments were passed
// 2- First item is not a valid task name
- if (empty($args) || (isset($args[0]) && strpos($args[0], '-') !== false)) {
+ if (empty($args) || ! isset($this->_tasks[$this->_processTaskName($args[0])])) {
array_unshift($args, 'help');
}
// Process all sent arguments
- $processedArgs = $this->_processArguments($args);
+ $processedArgs = $this->_processArguments($args);
try {
// Handle possible multiple tasks on a single command
diff --git a/lib/Doctrine/ORM/Tools/Cli/AbstractPrinter.php b/lib/Doctrine/ORM/Tools/Cli/Printers/AbstractPrinter.php
similarity index 89%
rename from lib/Doctrine/ORM/Tools/Cli/AbstractPrinter.php
rename to lib/Doctrine/ORM/Tools/Cli/Printers/AbstractPrinter.php
index f5d6e7a1b..4a9416fe6 100644
--- a/lib/Doctrine/ORM/Tools/Cli/AbstractPrinter.php
+++ b/lib/Doctrine/ORM/Tools/Cli/Printers/AbstractPrinter.php
@@ -19,7 +19,9 @@
* .
*/
-namespace Doctrine\ORM\Tools\Cli;
+namespace Doctrine\ORM\Tools\Cli\Printers;
+
+use Doctrine\ORM\Tools\Cli\Style;
/**
* CLI Output Printer.
@@ -137,7 +139,7 @@ abstract class AbstractPrinter
}
/**
- * Writes to output stream the message, formatting it by applying the defined style.
+ * Writes to the output stream, formatting it by applying the defined style.
*
* @param string $message Message to be outputted
* @param mixed $style Optional style to be applied in message
@@ -149,6 +151,17 @@ abstract class AbstractPrinter
fwrite($this->_stream, $this->format($message, $style));
}
+ /**
+ * Writes a line to the output stream, formatting it by applying the defined style.
+ *
+ * @param string $message Message to be outputted
+ * @param mixed $style Optional style to be applied in message
+ */
+ public function writeln($message, $style = 'ERROR')
+ {
+ $this->write($message . PHP_EOL, $style);
+ }
+
/**
* Formats the given message with the defined style.
*
diff --git a/lib/Doctrine/ORM/Tools/Cli/Printer/AnsiColor.php b/lib/Doctrine/ORM/Tools/Cli/Printers/AnsiColorPrinter.php
similarity index 97%
rename from lib/Doctrine/ORM/Tools/Cli/Printer/AnsiColor.php
rename to lib/Doctrine/ORM/Tools/Cli/Printers/AnsiColorPrinter.php
index 5c434fceb..d4b016aaf 100644
--- a/lib/Doctrine/ORM/Tools/Cli/Printer/AnsiColor.php
+++ b/lib/Doctrine/ORM/Tools/Cli/Printers/AnsiColorPrinter.php
@@ -19,10 +19,9 @@
* .
*/
-namespace Doctrine\ORM\Tools\Cli\Printer;
+namespace Doctrine\ORM\Tools\Cli\Printers;
-use Doctrine\ORM\Tools\Cli\AbstractPrinter,
- Doctrine\ORM\Tools\Cli\Style;
+use Doctrine\ORM\Tools\Cli\Style;
/**
* CLI Output Printer for ANSI Color terminal
@@ -35,7 +34,7 @@ use Doctrine\ORM\Tools\Cli\AbstractPrinter,
* @author Jonathan Wage
* @author Roman Borschel
*/
-class AnsiColor extends AbstractPrinter
+class AnsiColorPrinter extends AbstractPrinter
{
/**
* @inheritdoc
diff --git a/lib/Doctrine/ORM/Tools/Cli/Printer/Normal.php b/lib/Doctrine/ORM/Tools/Cli/Printers/NormalPrinter.php
similarity index 90%
rename from lib/Doctrine/ORM/Tools/Cli/Printer/Normal.php
rename to lib/Doctrine/ORM/Tools/Cli/Printers/NormalPrinter.php
index 52eb8e4c6..d447413f5 100644
--- a/lib/Doctrine/ORM/Tools/Cli/Printer/Normal.php
+++ b/lib/Doctrine/ORM/Tools/Cli/Printers/NormalPrinter.php
@@ -19,10 +19,9 @@
* .
*/
-namespace Doctrine\ORM\Tools\Cli\Printer;
+namespace Doctrine\ORM\Tools\Cli\Printers;
-use Doctrine\ORM\Tools\Cli\AbstractPrinter,
- Doctrine\ORM\Tools\Cli\Style;
+use Doctrine\ORM\Tools\Cli\Style;
/**
* CLI Output Printer for Normal terminal
@@ -35,7 +34,7 @@ use Doctrine\ORM\Tools\Cli\AbstractPrinter,
* @author Jonathan Wage
* @author Roman Borschel
*/
-class Normal extends AbstractPrinter
+class NormalPrinter extends AbstractPrinter
{
/**
* @inheritdoc
diff --git a/lib/Doctrine/ORM/Tools/Cli/AbstractTask.php b/lib/Doctrine/ORM/Tools/Cli/Tasks/AbstractTask.php
similarity index 74%
rename from lib/Doctrine/ORM/Tools/Cli/AbstractTask.php
rename to lib/Doctrine/ORM/Tools/Cli/Tasks/AbstractTask.php
index a9108b085..25c233c01 100644
--- a/lib/Doctrine/ORM/Tools/Cli/AbstractTask.php
+++ b/lib/Doctrine/ORM/Tools/Cli/Tasks/AbstractTask.php
@@ -19,12 +19,21 @@
* .
*/
-namespace Doctrine\ORM\Tools\Cli;
+namespace Doctrine\ORM\Tools\Cli\Tasks;
+
+use Doctrine\ORM\Tools\Cli\Printers\AbstractPrinter;
/**
- * CLI Task.
+ * Base class for CLI Tasks.
* Provides basic methods and requires implementation of methods that
* each task should implement in order to correctly work.
+ *
+ * The following arguments are common to all tasks:
+ *
+ * Argument: --config=
+ * Description: Specifies the path to the configuration file to use. The configuration file
+ * can bootstrap an EntityManager as well as provide defaults for any cli
+ * arguments.
*
* @license http://www.opensource.org/licenses/lgpl-license.php LGPL
* @link www.doctrine-project.org
@@ -51,6 +60,8 @@ abstract class AbstractTask
*/
protected $_availableTasks;
+ protected $_em;
+
/**
* Defines a CLI Output Printer
*
@@ -148,13 +159,44 @@ abstract class AbstractTask
*
* @return boolean
*/
- abstract public function validate();
+ public function validate()
+ {
+ if ( ! isset($this->_arguments['config'])) {
+ if (file_exists('./cli-config.php')) {
+ require './cli-config.php';
+ } else {
+ $this->_printer->writeln('--config option or cli-config.php in the same directory required', 'ERROR');
+ return false;
+ }
+ } else {
+ require $this->_arguments['config'];
+ }
+
+ // $em and $args come from the config
+ if (isset($em)) {
+ $this->_em = $em;
+ }
+ if (isset($args)) {
+ // Merge arguments. Values specified via the CLI take preference.
+ $this->_arguments = array_merge($args, $this->_arguments);
+ }
+
+ return true;
+ }
/**
* Safely execution of task.
* Each CLI task should implement this as normal flow execution of
* what is supposed to do.
- *
*/
abstract public function run();
+
+ protected function _requireEntityManager()
+ {
+ if ( ! isset($this->_em)) {
+ $this->_printer->writeln('No EntityManager created in configuration but required by task ' . get_class($this), 'ERROR');
+ return false;
+ }
+ return true;
+ }
}
\ No newline at end of file
diff --git a/lib/Doctrine/ORM/Tools/Cli/Tasks/CreateMappingTask.php b/lib/Doctrine/ORM/Tools/Cli/Tasks/CreateMappingTask.php
new file mode 100644
index 000000000..2c8fda7a3
--- /dev/null
+++ b/lib/Doctrine/ORM/Tools/Cli/Tasks/CreateMappingTask.php
@@ -0,0 +1,14 @@
+.
*/
-namespace Doctrine\ORM\Tools\Cli\Task;
-
-use Doctrine\ORM\Tools\Cli\AbstractTask;
+namespace Doctrine\ORM\Tools\Cli\Tasks;
/**
* CLI Task to display available commands help
@@ -34,7 +32,7 @@ use Doctrine\ORM\Tools\Cli\AbstractTask;
* @author Jonathan Wage
* @author Roman Borschel
*/
-class Help extends AbstractTask
+class HelpTask extends AbstractTask
{
/**
* @inheritdoc
diff --git a/lib/Doctrine/ORM/Tools/Cli/Tasks/RunSqlTask.php b/lib/Doctrine/ORM/Tools/Cli/Tasks/RunSqlTask.php
new file mode 100644
index 000000000..0714116c6
--- /dev/null
+++ b/lib/Doctrine/ORM/Tools/Cli/Tasks/RunSqlTask.php
@@ -0,0 +1,48 @@
+getPrinter()->writeln('run-sql extended help.', 'INFO');
+ }
+
+ /**
+ * @inheritdoc
+ */
+ public function basicHelp()
+ {
+ $this->getPrinter()->writeln('run-sql basic help.', 'INFO');
+ }
+
+ /**
+ * Executes the task.
+ */
+ public function run()
+ {
+ $args = $this->getArguments();
+
+ if (isset($args['file'])) {
+ //TODO
+ } else if (isset($args['sql'])) {
+ if (preg_match('/^select/i', $args['sql'])) {
+ $stmt = $this->_em->getConnection()->execute($args['sql']);
+ var_dump($stmt->fetchAll(\Doctrine\DBAL\Connection::FETCH_ASSOC));
+ } else {
+ var_dump($this->_em->getConnection()->executeUpdate($args['sql']));
+ }
+ }
+ }
+}
\ No newline at end of file
diff --git a/lib/Doctrine/ORM/Tools/Cli/Tasks/SchemaToolTask.php b/lib/Doctrine/ORM/Tools/Cli/Tasks/SchemaToolTask.php
new file mode 100644
index 000000000..b5f70eeaf
--- /dev/null
+++ b/lib/Doctrine/ORM/Tools/Cli/Tasks/SchemaToolTask.php
@@ -0,0 +1,158 @@
+--classdir=
+ * Specifies the directory where to start looking for mapped classes.
+ * This argument is required when the annotation metadata driver is used,
+ * otherwise it has no effect.
+ *
+ * --dump-sql
+ * Specifies that instead of directly executing the SQL statements for creating the
+ * database schema, they should be printed to the standard output.
+ *
+ * --create
+ * Specifies that the schema of the classes should be created.
+ *
+ * --drop
+ * Specifies that the schema of the classes should be dropped.
+ *
+ * --update
+ * Specifies that the schema of the classes should be updated.
+ *
+ *
+ * @author robo
+ * @since 2.0
+ */
+class SchemaToolTask extends AbstractTask
+{
+ /**
+ * @inheritdoc
+ */
+ public function extendedHelp()
+ {
+ $this->getPrinter()->writeln('create-schema extended help.', 'INFO');
+ }
+
+ /**
+ * @inheritdoc
+ */
+ public function basicHelp()
+ {
+ $this->getPrinter()->writeln('create-schema basic help.', 'INFO');
+ }
+
+ /**
+ * @inheritdoc
+ */
+ public function validate()
+ {
+ if ( ! parent::validate()) {
+ return false;
+ }
+
+ $args = $this->getArguments();
+ $printer = $this->getPrinter();
+
+ if ( ! $this->_requireEntityManager()) {
+ return false;
+ }
+
+ $isCreate = isset($args['create']);
+ $isDrop = isset($args['drop']);
+ $isUpdate = isset($args['update']);
+
+ if ( ! ($isCreate ^ $isDrop ^ $isUpdate)) {
+ $printer->writeln("One of --create, --drop or --update required, and only one.", 'ERROR');
+ return false;
+ }
+
+ if ($this->_em->getConfiguration()->getMetadataDriverImpl() instanceof \Doctrine\ORM\Mapping\Driver\AnnotationDriver
+ && ! isset($args['classdir'])) {
+ $printer->writeln("The supplied configuration uses the annotation metadata driver."
+ . " The 'classdir' argument is required for this driver.", 'ERROR');
+ return false;
+ }
+
+ return true;
+ }
+
+ /**
+ * Executes the task.
+ */
+ public function run()
+ {
+ $args = $this->getArguments();
+
+ $isCreate = isset($args['create']);
+ $isDrop = isset($args['drop']);
+ $isUpdate = isset($args['update']);
+
+ $cmf = $this->_em->getMetadataFactory();
+ $driver = $this->_em->getConfiguration()->getMetadataDriverImpl();
+
+ $classes = array();
+
+ if ($driver instanceof \Doctrine\ORM\Mapping\Driver\AnnotationDriver) {
+ $iter = new \FilesystemIterator($args['classdir']);
+
+ $declared = get_declared_classes();
+ foreach ($iter as $item) {
+ $baseName = $item->getBaseName();
+ if ($baseName[0] == '.') {
+ continue;
+ }
+ require_once $item->getPathName();
+ }
+ $declared = array_diff(get_declared_classes(), $declared);
+
+ foreach ($declared as $className) {
+ if ( ! $driver->isTransient($className)) {
+ $classes[] = $cmf->getMetadataFor($className);
+ }
+ }
+ } else {
+ $driver->preload();
+ $classes = $cmf->getLoadedMetadata();
+ }
+
+ $printer = $this->getPrinter();
+ $tool = new SchemaTool($this->_em);
+
+ if ($isCreate) {
+ if (isset($args['dump-sql'])) {
+ foreach ($tool->getCreateSchemaSql($classes) as $sql) {
+ $printer->writeln($sql, 'NONE');
+ }
+ } else {
+ $printer->writeln('Creating database schema...', 'INFO');
+ $tool->createSchema($classes);
+ $printer->write('Database schema created successfully.' . PHP_EOL, 'INFO');
+ }
+ } else if ($isDrop) {
+ if (isset($args['dump-sql'])) {
+ foreach ($tool->getDropSchemaSql($classes) as $sql) {
+ $printer->writeln($sql, 'NONE');
+ }
+ } else {
+ $printer->writeln('Dropping database schema...', 'INFO');
+ $tool->dropSchema($classes);
+ $printer->writeln('Database schema dropped successfully.', 'INFO');
+ }
+ } else if ($isUpdate) {
+ //TODO
+ $printer->writeln('--update not yet implemented.', 'COMMENT');
+ }
+ }
+}
\ No newline at end of file
diff --git a/lib/Doctrine/ORM/Tools/Cli/Task/Version.php b/lib/Doctrine/ORM/Tools/Cli/Tasks/VersionTask.php
similarity index 96%
rename from lib/Doctrine/ORM/Tools/Cli/Task/Version.php
rename to lib/Doctrine/ORM/Tools/Cli/Tasks/VersionTask.php
index b7ae5b7ed..2b0008441 100644
--- a/lib/Doctrine/ORM/Tools/Cli/Task/Version.php
+++ b/lib/Doctrine/ORM/Tools/Cli/Tasks/VersionTask.php
@@ -19,9 +19,7 @@
* .
*/
-namespace Doctrine\ORM\Tools\Cli\Task;
-
-use Doctrine\ORM\Tools\Cli\AbstractTask;
+namespace Doctrine\ORM\Tools\Cli\Tasks;
/**
* CLI Task to display the doctrine version
@@ -34,7 +32,7 @@ use Doctrine\ORM\Tools\Cli\AbstractTask;
* @author Jonathan Wage
* @author Roman Borschel
*/
-class Version extends AbstractTask
+class VersionTask extends AbstractTask
{
/**
* @inheritdoc
diff --git a/lib/Doctrine/ORM/Tools/SchemaTool.php b/lib/Doctrine/ORM/Tools/SchemaTool.php
index ebe49f48b..75c3e593f 100644
--- a/lib/Doctrine/ORM/Tools/SchemaTool.php
+++ b/lib/Doctrine/ORM/Tools/SchemaTool.php
@@ -22,17 +22,17 @@
namespace Doctrine\ORM\Tools;
use Doctrine\DBAL\Types\Type,
- Doctrine\ORM\EntityManager;
+ Doctrine\ORM\EntityManager,
+ Doctrine\ORM\Internal\CommitOrderCalculator;
/**
- * The SchemaTool is a tool to create and/or drop database schemas based on
+ * The SchemaTool is a tool to create/drop/update database schemas based on
* ClassMetadata class descriptors.
*
* @author Roman Borschel
* @license http://www.opensource.org/licenses/lgpl-license.php LGPL
* @link www.doctrine-project.org
* @since 2.0
- * @version $Revision: 4805 $
*/
class SchemaTool
{
@@ -382,8 +382,9 @@ class SchemaTool
public function getDropSchemaSql(array $classes)
{
$sql = array();
- $commitOrder = $classes; //FIXME: get real commit order!!
+ $commitOrder = $this->_getCommitOrder($classes);
+
// Drop tables in reverse commit order
for ($i = count($commitOrder) - 1; $i >= 0; --$i) {
$class = $commitOrder[$i];
@@ -393,6 +394,8 @@ class SchemaTool
$sql[] = $this->_platform->getDropTableSql($class->getTableName());
}
+ //TODO: Drop other schema elements, like sequences etc.
+
return $sql;
}
@@ -529,4 +532,26 @@ class SchemaTool
return $sql;
}
+
+ private function _getCommitOrder(array $classes)
+ {
+ $calc = new CommitOrderCalculator;
+
+ // Calculate dependencies
+ foreach ($classes as $class) {
+ $calc->addClass($class);
+ foreach ($class->associationMappings as $assoc) {
+ if ($assoc->isOwningSide) {
+ $targetClass = $this->_em->getClassMetadata($assoc->targetEntityName);
+ if ( ! $calc->hasClass($targetClass->name)) {
+ $calc->addClass($targetClass);
+ }
+ // add dependency ($targetClass before $class)
+ $calc->addDependency($targetClass, $class);
+ }
+ }
+ }
+
+ return $calc->getCommitOrder();
+ }
}
diff --git a/lib/Doctrine/ORM/UnitOfWork.php b/lib/Doctrine/ORM/UnitOfWork.php
index f2f7810db..590866a97 100644
--- a/lib/Doctrine/ORM/UnitOfWork.php
+++ b/lib/Doctrine/ORM/UnitOfWork.php
@@ -1290,7 +1290,7 @@ class UnitOfWork implements PropertyChangedListener
$managedCopyVersion = $class->reflFields[$class->versionField]->getValue($managedCopy);
$entityVersion = $class->reflFields[$class->versionField]->getValue($entity);
// Throw exception if versions dont match.
- if ($managedCopyVersion != $entity) {
+ if ($managedCopyVersion != $entityVersion) {
throw OptimisticLockException::versionMismatch();
}
}
diff --git a/tests/Doctrine/Tests/ORM/Mapping/MappingDriverTest.php b/tests/Doctrine/Tests/ORM/Mapping/MappingDriverTest.php
index a129276bb..78db0af2a 100644
--- a/tests/Doctrine/Tests/ORM/Mapping/MappingDriverTest.php
+++ b/tests/Doctrine/Tests/ORM/Mapping/MappingDriverTest.php
@@ -41,7 +41,7 @@ class MappingDriverTest extends \Doctrine\Tests\OrmTestCase
public function testXmlPreloadMode()
{
$className = 'Doctrine\Tests\ORM\Mapping\User';
- $xmlDriver = new XmlDriver(__DIR__ . DIRECTORY_SEPARATOR . 'xml');
+ $xmlDriver = new XmlDriver(__DIR__ . DIRECTORY_SEPARATOR . 'xml', XmlDriver::PRELOAD);
$class = new ClassMetadata($className);
$classNames = $xmlDriver->preload();
@@ -57,7 +57,7 @@ class MappingDriverTest extends \Doctrine\Tests\OrmTestCase
public function testYamlPreloadMode()
{
$className = 'Doctrine\Tests\ORM\Mapping\User';
- $yamlDriver = new YamlDriver(__DIR__ . DIRECTORY_SEPARATOR . 'yaml');
+ $yamlDriver = new YamlDriver(__DIR__ . DIRECTORY_SEPARATOR . 'yaml', YamlDriver::PRELOAD);
$class = new ClassMetadata($className);
$classNames = $yamlDriver->preload();
diff --git a/tools/sandbox/cli-config.php b/tools/sandbox/cli-config.php
new file mode 100644
index 000000000..fc638f20b
--- /dev/null
+++ b/tools/sandbox/cli-config.php
@@ -0,0 +1,14 @@
+setMetadataCacheImpl(new \Doctrine\Common\Cache\ArrayCache);
+$connectionOptions = array(
+ 'driver' => 'pdo_sqlite',
+ 'path' => 'database.sqlite'
+);
+
+$em = \Doctrine\ORM\EntityManager::create($connectionOptions, $config);
+
+$args = array(
+ 'classdir' => './Entities'
+);
\ No newline at end of file
diff --git a/tools/sandbox/config.php b/tools/sandbox/config.php
index 162aa0d7a..272cb22d6 100644
--- a/tools/sandbox/config.php
+++ b/tools/sandbox/config.php
@@ -6,9 +6,17 @@ $classLoader = new \Doctrine\Common\ClassLoader();
$classLoader->setBasePath('Doctrine', realpath(__DIR__ . '/../../lib'));
$classLoader->setBasePath('Entities', __DIR__);
-$config = new \Doctrine\ORM\Configuration();
-$config->setMetadataCacheImpl(new \Doctrine\ORM\Cache\ArrayCache);
-$config->setMetadataDriverImpl(new \Doctrine\ORM\Mapping\Driver\YamlDriver(__DIR__ . '/schema'));
+$config = new \Doctrine\ORM\Configuration;
+$cache = new \Doctrine\Common\Cache\ApcCache;
+$config->setMetadataCacheImpl($cache);
+$config->setQueryCacheImpl($cache);
+
+# EXAMPLE FOR YAML DRIVER
+#$config->setMetadataDriverImpl(new \Doctrine\ORM\Mapping\Driver\YamlDriver(__DIR__ . '/yaml'));
+
+# EXAMPLE FOR XML DRIVER
+#$config->setMetadataDriverImpl(new \Doctrine\ORM\Mapping\Driver\YamlDriver(__DIR__ . '/xml'));
+
$eventManager = new \Doctrine\Common\EventManager();
$connectionOptions = array(
'driver' => 'pdo_sqlite',
diff --git a/tools/sandbox/doctrine b/tools/sandbox/doctrine
new file mode 100755
index 000000000..87f76e486
--- /dev/null
+++ b/tools/sandbox/doctrine
@@ -0,0 +1,10 @@
+#!/usr/bin/env php
+setBasePath('Doctrine', __DIR__ . '/../../lib');
+
+$cli = new \Doctrine\ORM\Tools\Cli();
+$cli->run($_SERVER['argv']);
diff --git a/lib/Doctrine/ORM/PersistentCollection.php b/lib/Doctrine/ORM/PersistentCollection.php
index aaa5d9d06..b98d1853c 100644
--- a/lib/Doctrine/ORM/PersistentCollection.php
+++ b/lib/Doctrine/ORM/PersistentCollection.php
@@ -82,7 +82,7 @@ final class PersistentCollection implements \Doctrine\Common\Collections\Collect
private $_backRefFieldName;
/**
* The class descriptor of the owning entity.
*/
private $_typeClass;
@@ -124,7 +124,8 @@ final class PersistentCollection implements \Doctrine\Common\Collections\Collect
/**
* INTERNAL:
- * Sets the collection owner. Used (only?) during hydration.
+ * Sets the collection's owning entity together with the AssociationMapping that
+ * describes the association between the owner and the elements of the collection.
*
* @param object $entity
* @param AssociationMapping $assoc
@@ -168,7 +169,8 @@ final class PersistentCollection implements \Doctrine\Common\Collections\Collect
/**
* INTERNAL:
- * Adds an element to a collection during hydration.
+ * Adds an element to a collection during hydration. This will automatically
+ * complete bidirectional associations.
*
* @param mixed $element The element to add.
*/
@@ -198,9 +200,23 @@ final class PersistentCollection implements \Doctrine\Common\Collections\Collect
* @param mixed $key The key to set.
* $param mixed $value The element to set.
*/
- public function hydrateSet($key, $value)
+ public function hydrateSet($key, $element)
{
- $this->_coll->set($key, $value);
+ $this->_coll->set($key, $element);
+ // If _backRefFieldName is set, then the association is bidirectional
+ // and we need to set the back reference.
+ if ($this->_backRefFieldName) {
+ // Set back reference to owner
+ if ($this->_association->isOneToMany()) {
+ // OneToMany
+ $this->_typeClass->reflFields[$this->_backRefFieldName]
+ ->setValue($element, $this->_owner);
+ } else {
+ // ManyToMany
+ $this->_typeClass->reflFields[$this->_backRefFieldName]
+ ->getValue($element)->set($key, $this->_owner);
+ }
+ }
}
/**
@@ -307,9 +323,9 @@ final class PersistentCollection implements \Doctrine\Common\Collections\Collect
}
/**
+ * Sets the initialized flag of the collection, forcing it into that state.
*
- * @param $bool
- * @return unknown_type
+ * @param boolean $bool
*/
public function setInitialized($bool)
{
@@ -82,7 +82,7 @@ final class PersistentCollection implements \Doctrine\Common\Collections\Collect
private $_backRefFieldName;
/**
- * The class descriptor of the owning entity.
+ * The class descriptor of the collection's entity type.
*/
private $_typeClass;